﻿var debug = false;
var scriptPath = File($.fileName).path;
if(File(scriptPath+"/debug.txt").exists) debug=true;

var svgExportOptions = new ExportOptionsSVG();
svgExportOptions.fontSubsetting = SVGFontSubsetting.None;
svgExportOptions.coordinatePrecision = 1;
svgExportOptions.embedRasterImage = true;

var pngExportOptions = new ExportOptionsPNG24();

var Layer = function(){}
var ArtLayer = function(){}
var Document = function(){}
var LayerSet = function(){}

var mapID = {};

var xmlHeader ='<?xml version="1.0" encoding="UTF-8" ?>';

app.coordinateSystem = CoordinateSystem.DOCUMENTCOORDINATESYSTEM;
app.userInteractionLevel = UserInteractionLevel.DONTDISPLAYALERTS;

/**

JSZip - A Javascript class for generating Zip files
<http://jszip.stuartk.co.uk>

(c) 2009 Stuart Knightley <stuart [at] stuartk.co.uk>
Licenced under the GPLv3 and the MIT licences

Usage:
   zip = new JSZip();
   zip.add("hello.txt", "Hello, World!").add("tempfile", "nothing");
   zip.folder("images").add("smile.gif", base64Data, {base64: true});
   zip.add("Xmas.txt", "Ho ho ho !", {date : new Date("December 25, 2007 00:00:01")});
   zip.remove("tempfile");

   base64zip = zip.generate();

**/

function JSZip(compression)
{
   // default : no compression
   this.compression = (compression || "STORE").toUpperCase();
   this.files = [];

   // Where we are in the hierarchy
   this.root = "";

   // Default properties for a new file
   this.d = {
      base64: false,
      binary: false,
      dir: false,
      date: null
   };

   if (!JSZip.compressions[this.compression]) {
      throw compression + " is not a valid compression method !";
   }
}

/**
 * Add a file to the zip file
 * @param   name  The name of the file
 * @param   data  The file data, either raw or base64 encoded
 * @param   o     File options
 * @return  this JSZip object
 */
JSZip.prototype.add = function(name, data, o)
{
   o = o || {};
   name = this.root+name;

   if (o.base64 === true && o.binary == null) o.binary = true;

   for (var opt in this.d)
   {
      o[opt] = o[opt] || this.d[opt];
   }

   o.date = o.date || new Date();
   var dosTime, dosDate;

   dosTime = o.date.getHours();
   dosTime = dosTime << 6;
   dosTime = dosTime | o.date.getMinutes();
   dosTime = dosTime << 5;
   dosTime = dosTime | o.date.getSeconds() / 2;

   dosDate = o.date.getFullYear() - 1980;
   dosDate = dosDate << 4;
   dosDate = dosDate | (o.date.getMonth() + 1);
   dosDate = dosDate << 5;
   dosDate = dosDate | o.date.getDate();

   if (o.base64 === true) data = JSZipBase64.decode(data);
   // decode UTF-8 strings if we are dealing with text data
   if(o.binary === false) data = this.utf8encode(data);


   var compression    = JSZip.compressions[this.compression];
   var compressedData = compression.compress(data);

   var header = "";

   // version needed to extract
   header += "\x0A\x00";
   // general purpose bit flag
   header += "\x00\x00";
   // compression method
   header += compression.magic;
   // last mod file time
   header += this.decToHex(dosTime, 2);
   // last mod file date
   header += this.decToHex(dosDate, 2);
   // crc-32
   header += this.decToHex(this.crc32(data), 4);
   // compressed size
   header += this.decToHex(compressedData.length, 4);
   // uncompressed size
   header += this.decToHex(data.length, 4);
   // file name length
   header += this.decToHex(name.length, 2);
   // extra field length
   header += "\x00\x00";

   // file name

   this.files[name] = {header: header, data: compressedData, dir: o.dir};

   return this;
};

/**
 * Add a directory to the zip file
 * @param   name  The name of the directory to add
 * @return  JSZip object with the new directory as the root
 */
JSZip.prototype.folder = function(name)
{
   // Check the name ends with a /
   if (name.substr(-1) != "/") name += "/";

   // Does this folder already exist?
   if (typeof this.files[name] === "undefined") this.add(name, '', {dir:true});

   // Allow chaining by returning a new object with this folder as the root
   var ret = this.clone();
   ret.root = this.root+name;
   return ret;
};

/**
 * Compare a string or regular expression against all of the filenames and
 * return an informational object for each that matches.
 * @param   string/regex The regular expression to test against
 * @return  An array of objects representing the matched files. In the form
 *          {name: "filename", data: "file data", dir: true/false}
 */
JSZip.prototype.find = function(needle)
{
   var result = [], re;
   if (typeof needle === "string")
   {
      re = new RegExp("^"+needle+"$");
   }
   else
   {
      re = needle;
   }

   for (var filename in this.files)
   {
      if (re.test(filename))
      {
         var file = this.files[filename];
         result.push({name: filename, data: file.data, dir: !!file.dir});
      }
   }

   return result;
};

/**
 * Delete a file, or a directory and all sub-files, from the zip
 * @param   name  the name of the file to delete
 * @return  this JSZip object
 */
JSZip.prototype.remove = function(name)
{
   var file = this.files[name];
   if (!file)
   {
      // Look for any folders
      if (name.substr(-1) != "/") name += "/";
      file = this.files[name];
   }

   if (file)
   {
      if (name.match("/") === null)
      {
         // file
         delete this.files[name];
      }
      else
      {
         // folder
         var kids = this.find(new RegExp("^"+name));
         for (var i = 0; i < kids.length; i++)
         {
            if (kids[i].name == name)
            {
               // Delete this folder
               delete this.files[name];
            }
            else
            {
               // Remove a child of this folder
               this.remove(kids[i].name);
            }
         }
      }
   }

   return this;
};

/**
 * Generate the complete zip file
 * @return  A base64 encoded string of the zip file
 */
JSZip.prototype.generate = function(asBytes)
{
   asBytes = asBytes || false;

   // The central directory, and files data
   var directory = [], files = [], fileOffset = 0;

   for (var name in this.files)
   {
      if( !this.files.hasOwnProperty(name) ) { continue; }

      var fileRecord = "", dirRecord = "";
      fileRecord = "\x50\x4b\x03\x04" + this.files[name].header + name + this.files[name].data;

      dirRecord = "\x50\x4b\x01\x02" +
      // version made by (00: DOS)
      "\x14\x00" +
      // file header (common to file and central directory)
      this.files[name].header +
      // file comment length
      "\x00\x00" +
      // disk number start
      "\x00\x00" +
      // internal file attributes TODO
      "\x00\x00" +
      // external file attributes
      (this.files[name].dir===true?"\x10\x00\x00\x00":"\x00\x00\x00\x00")+
      // relative offset of local header
      this.decToHex(fileOffset, 4) +
      // file name
      name;

      fileOffset += fileRecord.length;

      files.push(fileRecord);
      directory.push(dirRecord);
   }

   var fileData = files.join("");
   var dirData = directory.join("");

   var dirEnd = "";

   // end of central dir signature
   dirEnd = "\x50\x4b\x05\x06" +
   // number of this disk
   "\x00\x00" +
   // number of the disk with the start of the central directory
   "\x00\x00" +
   // total number of entries in the central directory on this disk
   this.decToHex(files.length, 2) +
   // total number of entries in the central directory
   this.decToHex(files.length, 2) +
   // size of the central directory   4 bytes
   this.decToHex(dirData.length, 4) +
   // offset of start of central directory with respect to the starting disk number
   this.decToHex(fileData.length, 4) +
   // .ZIP file comment length
   "\x00\x00";

   var zip = fileData + dirData + dirEnd;
   return (asBytes) ? zip : JSZipBase64.encode(zip);

};

/*
 * Compression methods
 * This object is filled in as follow :
 * name : {
 *    magic // the 2 bytes indentifying the compression method
 *    compress // function, take the uncompressed content and return it compressed.
 * }
 *
 * STORE is the default compression method, so it's included in this file.
 * Other methods should go to separated files : the user wants modularity.
 */
JSZip.compressions = {
   "STORE" : {
      magic : "\x00\x00",
      compress : function (content) {
         return content; // no compression
      }
   }
};

// Utility functions

JSZip.prototype.decToHex = function(dec, bytes)
{
   var hex = "";
   for(var i=0;i<bytes;i++) {
      hex += String.fromCharCode(dec&0xff);
      dec=dec>>>8;
   }
   return hex;
};

/**
*
*  Javascript crc32
*  http://www.webtoolkit.info/
*
**/

JSZip.prototype.crc32 = function(str, crc)
{

   if (str === "") return "\x00\x00\x00\x00";

   var table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

   if (typeof(crc) == "undefined") { crc = 0; }
   var x = 0;
   var y = 0;

   crc = crc ^ (-1);
   for( var i = 0, iTop = str.length; i < iTop; i++ ) {
      y = ( crc ^ str.charCodeAt( i ) ) & 0xFF;
      x = "0x" + table.substr( y * 9, 8 );
      crc = ( crc >>> 8 ) ^ x;
   }

   return crc ^ (-1);

};

// Inspired by http://my.opera.com/GreyWyvern/blog/show.dml/1725165
JSZip.prototype.clone = function()
{
   var newObj = new JSZip();
   for (var i in this)
   {
      if (typeof this[i] !== "function")
      {
         newObj[i] = this[i];
      }
   }
   return newObj;
};

JSZip.prototype.utf8encode = function(input)
{
   input = encodeURIComponent(input);
   input = input.replace(/%.{2,2}/g, function(m) {
      var hex = m.substring(1);
      return String.fromCharCode(parseInt(hex,16));
   });
   return input;
};

/**
*
*  Base64 encode / decode
*  http://www.webtoolkit.info/
*
*  Hacked so that it doesn't utf8 en/decode everything
**/

var JSZipBase64 = function() {
   // private property
   var _keyStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";

   return {
      // public method for encoding
      encode : function(input, utf8) {
         var output = "";
         var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
         var i = 0;

         while (i < input.length) {

            chr1 = input.charCodeAt(i++);
            chr2 = input.charCodeAt(i++);
            chr3 = input.charCodeAt(i++);

            enc1 = chr1 >> 2;
            enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
            enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
            enc4 = chr3 & 63;

            if (isNaN(chr2)) {
               enc3 = enc4 = 64;
            } else if (isNaN(chr3)) {
               enc4 = 64;
            }

            output = output +
            _keyStr.charAt(enc1) + _keyStr.charAt(enc2) +
            _keyStr.charAt(enc3) + _keyStr.charAt(enc4);

         }

         return output;
      },

      // public method for decoding
      decode : function(input, utf8) {
         var output = "";
         var chr1, chr2, chr3;
         var enc1, enc2, enc3, enc4;
         var i = 0;

         input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");

         while (i < input.length) {

            enc1 = _keyStr.indexOf(input.charAt(i++));
            enc2 = _keyStr.indexOf(input.charAt(i++));
            enc3 = _keyStr.indexOf(input.charAt(i++));
            enc4 = _keyStr.indexOf(input.charAt(i++));

            chr1 = (enc1 << 2) | (enc2 >> 4);
            chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
            chr3 = ((enc3 & 3) << 6) | enc4;

            output = output + String.fromCharCode(chr1);

            if (enc3 != 64) {
               output = output + String.fromCharCode(chr2);
            }
            if (enc4 != 64) {
               output = output + String.fromCharCode(chr3);
            }

         }

         return output;

      }
   };
}();

JSZip.prototype.addExistingFile = function(/*File*/f, newName)
//--------------------------------------
{
var contents = false;
 
if( f.constructor == File && f.exists && (f.encoding = 'BINARY') && f.open('r') )
    {
    contents = f.read();
    f.close();
    return this.add(
        newName||f.name,
        contents,
        {binary: true, date: f.modified}
        );
    }
throw new Error("Unable to open the file "+f);
}

JSZip.prototype.addExistingFolder = function(/*folder*/f)
//--------------------------------------
{
var contents = false;

var zipFolder = this.folder(f.displayName);

//add files
var allFiles = f.getFiles();
for(var i=0;i<allFiles.length;i++)
{
    var currFile = allFiles[i];
    if(currFile.constructor.name=="File")
    zipFolder.addExistingFile(currFile,currFile.displayName);
    else
    zipFolder.addExistingFolder(currFile);
}
return zipFolder;

}

Math.uuid = (function() {
  // Private array of chars to use
  var CHARS = '0123456789abcdefghijklmnopqrstuvwxyz'.split(''); 
 
  return function (len, radix) {
    var chars = CHARS, uuid = [];
    radix = radix || chars.length;
 
    if (len) {
      // Compact form
      for (var i = 0; i < len; i++) uuid[i] = chars[0 | Math.random()*radix];
    } else {
      // rfc4122, version 4 form
      var r;
 
      // rfc4122 requires these characters
      uuid[8] = uuid[13] = uuid[18] = uuid[23] = '-';
      uuid[14] = '4';
 
      // Fill in random data.  At i==19 set the high bits of clock sequence as
      // per rfc4122, sec. 4.1.5
      for (var i = 0; i < 36; i++) {
        if (!uuid[i]) {
          r = 0 | Math.random()*16;
          uuid[i] = chars[(i == 19) ? (r & 0x3) | 0x8 : r];
        }
      }
    }
 
    return uuid.join('');
  };
})();

Folder.prototype.removeAll = function()
{
        var allFiles = this.getFiles();
        for(var i=0;i<allFiles.length;i++)
        {
                if(allFiles[i].constructor.name == "Folder") 
                    allFiles[i].removeAll();
                    
               allFiles[i].remove();     
        }
    
    this.remove();
}

String.prototype.encoded=function()
{
    return this.replace(/&/g, "&amp;").replace(/"/g, "&quot;").replace(/>/g, "&gt;").replace(/</g, "&lt;").replace(/'/g, "&apos;"); 
}


File.prototype.partname = function()
{
	var dotPosition=this.displayName.lastIndexOf(".");
    if(dotPosition!=-1)
	return this.displayName.substring(0,dotPosition);
    else return this.displayName;
}  

Folder.prototype.createOnce = function() {
    if (!this.exists) this.create();
    return this;
}

File.prototype.renameOverwrite = function(newName)
{
    var targetFile = File(this.parent+"/"+newName);
    if(targetFile.exists) targetFile.remove();
    this.rename(newName);
}

Array.prototype.lastItem = function() {
    if (this.length) return this[this.length - 1];
}
    
var myProgressBarWindow;
var myProgressBar;
var totalLayers;
    
var outputFolder = undefined;
   
try{    
    main();
}catch(e){log(e); alert(e)}


function main()
{
    
    if(!app.documents.length) return;
    
    outputFolder = Folder.selectDialog("Please select output folder");
    if(!outputFolder) return;
    
   
    var docRef = app.activeDocument;  
    
    totalLayers = docRef.pageItems.length;
    
    myProgressBarWindow = new Window('palette', 'Progress', [150, 150, 600, 300]);
    myProgressBar = myProgressBarWindow.add('progressbar', [20, 35, 410, 60], 0,  totalLayers);
    myProgressBar.value=0;	
    myProgressBarWindow.show();

	mapID = {};
    
    var imagesFolder = Folder(outputFolder+"/images").createOnce();
    
    var aiFile = docRef.fullName;
    
    
    //sort layers, sublayers and page items into tree
    var artboardItems = [];
    
for(var j=0;j<docRef.artboards.length;j++)
{
    var allItems = [];
for(var i=0;i<docRef.layers.length;i++)
{
    var currLayer = docRef.layers[i];
   allItems.push({itemRef:currLayer,
       name:currLayer.name,
       type:currLayer.typename,
       locked:currLayer.locked,
       visible:currLayer.visible,
       zIndex:currLayer.absoluteZOrderPosition,
       children:getLayerChildren(currLayer,docRef.artboards[j].artboardRect)
       });
   currLayer.locked = false;
   currLayer.visible = true;
}
allItems.sort(sortByZIndex);



    var xmlData = XML("<screen/>");
    xmlData.appendChild(XML('<name>'+docRef.artboards[j].name+'</name>'));
    xmlData.appendChild(XML('<date>'+currDate()+'</date>'));
    
    var artRect = docRef.artboards[j].artboardRect;
    var artboardWidth = Math.round(artRect[2]-artRect[0]);
    var artboardHeight = Math.round(artRect[1]-artRect[3]);
    docRef.artboards.setActiveArtboardIndex(j);
    xmlData.appendChild(XML('<width>'+Math.round(artboardWidth)+'</width>'));
    xmlData.appendChild(XML('<height>'+Math.round(artboardHeight)+'</height>'));
    xmlData.appendChild(XML('<orientation>'+getOrientation(artRect)+'</orientation>'));
    
      
    artStartX = artRect[0];
    artStartY = artRect[1];
  
    exportNodeTree(allItems,xmlData);
  
    var xmlFolder = Folder(outputFolder+"/screens").createOnce();
    var xmlFile = File(xmlFolder+"/"+docRef.artboards[j].name+".xml");
    xmlFile.encoding = "UTF-8";
    xmlFile.open("w");
    xmlFile.write(xmlData.toXMLString());
    xmlFile.close();
    
    

}




    


 
    /*
    The result (images folder and the xml generated) should be included in the zip file attached
    the images folder in the root level of that zip file and the xml inside the 'screens' folder then,
    that zip file should be renamed to .vp
    */

    var zip = new JSZip();
    var baseFolder = Folder(scriptPath+"/base");
    var allBaseFiles = baseFolder.getFiles();
    for(var i=0;i<allBaseFiles.length;i++)
    {
        var currFile = allBaseFiles[i];
        if(currFile.constructor.name=="File")
            zip.addExistingFile(currFile,currFile.displayName);
        else
            zip.addExistingFolder(currFile);
    }
 
    zip.addExistingFolder(imagesFolder);
    zip.addExistingFolder(xmlFolder);

    var content = zip.generate(true); // asBytes

    var f = File(outputFolder+'/'+aiFile.partname()+'.vp');
    f.encoding = 'BINARY';
    if( f.open('w') )
    {
        f.write(content);
        f.close();
    }

    imagesFolder.removeAll();
    xmlFolder.removeAll();
    myProgressBarWindow.close();
}

function exportNodeTree(allItems,xmlData)
{
            var xmlNode = XML('<item-group><style name="LnFCanvas"><DimensionStyle widthType="px" heightType="px" width="1440" height="1024" widthPercentage="0.0" heightPercentage="0.0" /><PositioningStyle left="0" top="0" pinLeft="none" pinTop="none" /><BackgroundStyle type="color" value="255r255g255b0a" /><TransparencyStyle value="0" /></style></item-group>');
                xmlNode.@visible = true;
                 var uuid = createNewUUID();
                 xmlNode.@id = uuid;
                 for(var i=0;i<allItems.length;i++)
                {
//                    myProgressBar.value++;
   //                 myProgressBarWindow.text = "Processing layer "+myProgressBar.value+" out of "+totalLayers;
                    exportNode(allItems[i],xmlNode,0,0);

                 }
                xmlData.appendChild(xmlNode);
}


function exportNode(itemData,xmlData,startX,startY)
{
    var ref = itemData.itemRef;
    try{
        switch(itemData.type)
        {
            case "Layer":
            
                for(var i=0;i<itemData.children.length;i++)
                {
                   // myProgressBar.value++;
                   // myProgressBarWindow.text = "Processing layer "+myProgressBar.value+" out of "+totalLayers;
                    exportNode(itemData.children[i],xmlData,startX,startY);
                 }
         
             
            break;
            
              
               case "GroupItem":
               myProgressBar.value++;
               myProgressBarWindow.text = "Processing item "+myProgressBar.value+" out of "+totalLayers;
             var xmlNode = XML('<group-container lockType="none" onTop="false" />');
         
                xmlNode.@visible = itemData.visible;
                 xmlNode.@hidden = !itemData.visible;
                var uuid = createNewUUID();
                 xmlNode.@id = uuid;
                 
              var styleNode = XML("<style/>");
            styleNode.@name = "LnFGroup";
             
            var coords = getVisibleCoords(ref);
           
            var posNode = XML("<PositioningStyle/>");

            posNode.@left = Math.round(coords.x-artStartX-startX);
            posNode.@top =  Math.round(artStartY-coords.y-startY);
            
            var startGrX = coords.x-artStartX;
            var startGrY = artStartY-coords.y;
            
            styleNode.appendChild(posNode);
            
            var dimNode = XML('<DimensionStyle widthType="px" heightType="px" widthPercentage="0.0" heightPercentage="0.0" autofit="false"/>');
            
                dimNode.@width = Math.round(coords.width);
                dimNode.@height = Math.round(coords.height);
        
                        
            styleNode.appendChild(dimNode);
             styleNode.appendChild(XML('<RotationStyle angle="0"/>'));
             
            xmlNode.appendChild(styleNode);
            
                for(var i=0;i<itemData.children.length;i++)
                {
                   // myProgressBar.value++;
                   // myProgressBarWindow.text = "Processing layer "+myProgressBar.value+" out of "+totalLayers;
                    exportNode(itemData.children[i],xmlNode,startGrX,startGrY);
                 }
              xmlData.appendChild(xmlNode);
             
            break;
            
            
            case "PageItem":
            case "PathItem":
            case "PlacedItem":
            case "PluginItem":
            case "GraphItem":
            case "RasterItem":
            case "SymbolItem":
            case "EmbedItem":
            case "CompoundPathItem":
            case "TextFrame":
            
            default:
                    myProgressBar.value++;
                    myProgressBarWindow.text = "Processing item "+myProgressBar.value+" out of "+totalLayers;
                    exportItemNode(itemData,xmlData,startX,startY);
                
            break;
           
        }
    }catch(e){log(e+": "+ref.name)}

}


function exportItemNode(itemData,xmlData,startX,startY)
{
    var ref = itemData.itemRef;
    if(ref.parent.typename=="CompoundPathItem") return;
    var docRef = app.activeDocument; 
    var aiFile = docRef.fullName;
    var imagesFolder = Folder(outputFolder+"/images").createOnce();
    var isVisible = itemData.visible;
    
  
 
var uuid = createNewUUID();
  // log(ref.typename);
    switch(ref.typename)
    {
    
        case "TextFrame":
        {
            if(ref.contents=="") break;
         
            var xmlNode = XML("<rich-text/>");
            xmlNode.@id = uuid;
            xmlNode.@hidden = !isVisible;
            xmlNode.@lockType = getLockType(ref);
            xmlNode.@onTop = false;
            xmlNode.@visible = isVisible;
          
            var text = removeInvalidCharacters(ref.contents.encoded());
        //    if(isAllCaps(ref))
       //         text = text.toUpperCase();
            xmlNode.appendChild(XML("<text>"+text+"</text>"));
           
            var styleNode = XML("<style/>");
            styleNode.@name = "LnFRichText";
             
            var coords = getVisibleCoords(ref);
         
            var posNode = XML("<PositioningStyle/>");

            posNode.@left = Math.round(coords.x-artStartX-startX);
            posNode.@top = Math.round(artStartY-coords.y-startY);
           
            styleNode.appendChild(posNode);
           
         
           
            
            
            
            var dimNode = XML('<DimensionStyle widthType="px" heightType="px" widthPercentage="0.0" heightPercentage="0.0" autofit="false"/>');
            
                dimNode.@width = Math.round(coords.width);
                dimNode.@height = Math.round(coords.height);
        
                        
            styleNode.appendChild(dimNode);
            
            styleNode.appendChild(XML('<BackgroundStyle type="transparent" />'));
            
            styleNode.appendChild(XML('<BorderStyle radius="0" leftColor="0r0g0b0a" topColor="0r0g0b0a" rightColor="0r0g0b0a" bottomColor="0r0g0b0a" leftStyle="none" topStyle="none" rightStyle="none" bottomStyle="none" leftWidth="1" topWidth="1" rightWidth="1" bottomWidth="1" />'));
            
            
                
            var textStyleNode = XML("<TextStyle/>");
          
            textStyleNode.@hAlign = getAlign(ref.textRange);
            textStyleNode.@vAlign = "left";
            var textColor = getColor(ref.textRange.fillColor);
            textStyleNode.@color = Math.round(textColor[0])+"r"+Math.round(textColor[1])+"g"+Math.round(textColor[2])+"b0a";
            
 
            textStyleNode.@decoration = getTextDecoration(ref.textRange);
              
        
            var leading =  (ref.textRange.characterAttributes.leading*72/96).toFixed(0);
          
            var fontSize= (ref.textRange.characterAttributes.size*72/96).toFixed(1);
            
              
            
            textStyleNode.@['line-height'] = leading;
            styleNode.appendChild(textStyleNode);
           // var jimFontScaleRatio= app.activeDocument.resolution/96;
            
            var fontNode = XML("<FontStyle/>");
            fontNode.@size= fontSize; //parseFloat(ref.textItem.size.as("px") * jimFontScaleRatio).toFixed(1);
            fontNode.@height=15;
          
           
                var currFont = ref.textRange.characterAttributes.textFont;
                fontNode.appendChild(XML("<family>"+currFont.family+"</family>"));
                fontNode.appendChild(XML('<style name="'+currFont.style+'" weight="'+getFontWeight(currFont.style)+'" italic="'+getItalic(currFont.style)+'"/>'));
          
              
            styleNode.appendChild(fontNode);
            styleNode.appendChild(XML("<TransparencyStyle value=\""+(100-ref.opacity).toFixed(0)+"\"/>"));
        
            styleNode.appendChild(XML('<PaddingStyle top="0" left="0" bottom="0" right="0"/>'));
            styleNode.appendChild(XML('<RotationStyle angle="0"/>'));
            styleNode.appendChild(XML('<ShadowStyle><Box enable="false"/><Text enable="false"/></ShadowStyle>'));
            styleNode.appendChild(XML('<MarginStyle top="0" left="0" bottom="0" right="0"/>'));
               
            xmlNode.appendChild(styleNode);
       
          
       
     
            var rangeListNode = XML("<rich-text-range-list/>");
         
            //loop start
            //get text ranges data
            
            var allRanges = [];
            
            for(var i=0;i<ref.textRanges.length;i++)
            try{
                    var currRange = ref.textRanges[i];
                   
                    var rangeObj = {};
                    rangeObj.leading = (currRange.characterAttributes.leading*72/96).toFixed(0);
                    rangeObj.fontSize= (currRange.characterAttributes.size*72/96).toFixed(1);
                    var currFont = currRange.characterAttributes.textFont;
               rangeObj.family = currFont.family;
               rangeObj.style = currFont.style;
                rangeObj.textColor = getColor(currRange.fillColor); //array
                rangeObj.decoration = getTextDecoration(currRange);
                rangeObj.hAlign = getAlign (currRange);
               
                if(!allRanges.length || rangeObj.toSource()!=allRanges.lastItem().rangeObj.toSource())
                    allRanges.push({rangeObj:rangeObj,start:i,end:i+1});
                  else
                        allRanges.lastItem().end = i+1;
                
            }catch(e){log(e);return;}
        
        
            for(var i=0;i<allRanges.length;i++)
            {
            //create nodes
               var currRange = allRanges[i];
               var rangeObj = currRange.rangeObj;
               
                    var rangeNode = XML('<text-range text-range-start="'+currRange.start+'" text-range-end="'+currRange.end+'" />');
                    var styleNode = XML("<style/>");
                    styleNode.@name = "LnFTextRange"; 

    
           
                    var fontNode = XML("<FontStyle/>");
                    fontNode.@size=rangeObj.fontSize;
                    fontNode.@height=15;
                    
                    
                    fontNode.appendChild(XML("<family>"+rangeObj.family+"</family>"));
                    fontNode.appendChild(XML('<style name="'+rangeObj.style+'" weight="'+getFontWeight(rangeObj.style)+'" italic="'+getItalic(rangeObj.style)+'"/>'));
          
                    styleNode.appendChild(fontNode);

                    var textStyleNode = XML("<TextStyle/>");
                    
                       textStyleNode.@hAlign = rangeObj.hAlign;
            textStyleNode.@vAlign = "left";
            var textColor = rangeObj.textColor;
            textStyleNode.@color = Math.round(textColor[0])+"r"+Math.round(textColor[1])+"g"+Math.round(textColor[2])+"b0a";
            
            
            
                     textStyleNode.@decoration = rangeObj.decoration;
              
        
                  
            
              
            
                    textStyleNode.@['line-height'] = rangeObj.leading;
                    styleNode.appendChild(textStyleNode);

                    rangeNode.appendChild(styleNode);
                    rangeListNode.appendChild(rangeNode);
               }
            //loop end
                
           
            xmlNode.appendChild(rangeListNode);
          
            xmlNode.appendChild(XML("<userID>"+calculateValidName(ref.name.encoded())+"</userID>"));
            xmlData.appendChild(xmlNode);
             
    /*
        <rich-text id="fd0e556e-e970-44d5-85ef-de50cac7d94b" hidden="false" lockType="none" onTop="false" visible="true">
            <text>TRAVELSHARE</text>
            <style name="LnFRichText">
              <DimensionStyle widthType="px" heightType="px" width="153" height="24" widthPercentage="0.0" heightPercentage="0.0" />
              <PositioningStyle left="0" top="85" pinLeft="center" pinTop="none" />
              <BackgroundStyle type="transparent" />
              <BorderStyle radius="0" leftColor="0r0g0b0a" topColor="0r0g0b0a" rightColor="0r0g0b0a" bottomColor="0r0g0b0a" leftStyle="none" topStyle="none" rightStyle="none" bottomStyle="none" leftWidth="1" topWidth="1" rightWidth="1" bottomWidth="1" />
              <TextStyle color="255r255g255b0a" hAlign="left" vAlign="left" decoration="none" line-height="0" />
              <FontStyle size="15.0" height="20">
                <family>Montserrat</family>
                <style name="SemiBold" weight="600" italic="normal" />
                <evc-file>Montserrat-SemiBold</evc-file>
              </FontStyle>
              <TransparencyStyle value="0" />
              <PaddingStyle top="0" left="0" bottom="0" right="0" />
              <RotationStyle angle="0" />
              <ShadowStyle>
                <Box enable="false" />
                <Text enable="false" />
              </ShadowStyle>
              <MarginStyle top="0" left="0" bottom="0" right="0" />
            </style>
            <rich-text-range-list>
              <text-range text-range-start="0" text-range-end="11">
                <style name="LnFTextRange">
                  <FontStyle size="15.0" height="20">
                    <family>Montserrat</family>
                    <style name="SemiBold" weight="600" italic="normal" />
                    <evc-file>Montserrat-SemiBold</evc-file>
                  </FontStyle>
                  <TextStyle color="255r255g255b0a" hAlign="left" vAlign="left" decoration="none" line-height="0" />
                </style>
              </text-range>
            </rich-text-range-list>
            <userID>TRAVELSHARE_2</userID>
          </rich-text>
          */
        }
        break;
        default:
        {
           docRef.selection = undefined;
           ref.selected = true;
            var targetSvgName = uuid+ ".svg"; 
            var targetPngName = uuid+ ".png"; 
            
            targetName = targetPngName;
            
            //save png
             var targetPngFile = File(imagesFolder+"/tmp.png");
             docRef.exportSelectionAsPNG(targetPngFile,pngExportOptions);
             targetPngFile.renameOverwrite(targetPngName);
             
             if(ref.typename!="PlacedItem")
             {
            var targetAiFile = File(imagesFolder+"/tmp.ai");
              
            docRef.exportSelectionAsAi(targetAiFile);
            docRef.selection = undefined;
            var dupRef = app.open(targetAiFile);
            
            var targetFile = File(imagesFolder+"/tmp.svg");
            dupRef.exportFile(targetFile,ExportType.SVG,svgExportOptions);
            targetFile.renameOverwrite(targetSvgName);
            
            
            
            dupRef.close(SaveOptions.DONOTSAVECHANGES);
            targetAiFile.remove();
            }
            docRef.selection = undefined;
            var xmlNode = XML("<image/>");

            xmlNode.@hidden = !isVisible;
            xmlNode.@id = uuid;
            xmlNode.@lockType = getLockType(itemData.locked);
            xmlNode.@onTop = false;
            xmlNode.@visible = isVisible;

            xmlNode.appendChild(XML('<image-name type="svg" preserveRatio="false">'+targetName+"</image-name>"));
            xmlNode.appendChild(XML("<original-name>"+targetName+"</original-name>"));
            var styleNode = XML("<style/>");
            styleNode.@name = "LnFImage"; //?

            var dimNode = XML('<DimensionStyle widthType="px" heightType="px" widthPercentage="0.0" heightPercentage="0.0"/>');
            var posNode = XML("<PositioningStyle/>");
            posNode.@pinLeft = "none";
            posNode.@pinTop = "none";
            
   
            
            
            posNode.@left = Math.round(ref.position[0]-artStartX-startX);
            posNode.@top = Math.round(artStartY-ref.position[1]-startY);
            
            dimNode.@width = Math.round(ref.width);
            dimNode.@height = Math.round(ref.height);

            styleNode.appendChild(dimNode);
            styleNode.appendChild(posNode);

            styleNode.appendChild(XML('<BorderStyle radius="0" leftColor="0r0g0b0a" topColor="0r0g0b0a" rightColor="0r0g0b0a" bottomColor="0r0g0b0a" leftStyle="none" topStyle="none" rightStyle="none" bottomStyle="none" leftWidth="1" topWidth="1" rightWidth="1" bottomWidth="1" />'));

            styleNode.appendChild(XML("<TransparencyStyle value=\""+(100-ref.opacity).toFixed(0)+"\"/>"));
            styleNode.appendChild(XML('<RotationStyle angle="0" />'));
            styleNode.appendChild(XML('<ShadowStyle><Box enable="false" /></ShadowStyle>'));
            styleNode.appendChild(XML('<SVGBackgroundStyle isSVG="true" bgColor="0r0g0b0a" hasColor="false" />'));
            styleNode.appendChild(XML('<MarginStyle top="0" left="0" bottom="0" right="0" />'));

            xmlNode.appendChild(styleNode);

            xmlNode.appendChild(XML("<userID>"+calculateValidName(ref.name.encoded())+"</userID>"));
            xmlData.appendChild(xmlNode);
           
            
            }
            break;
        }
        
    }



function getLockType(isLocked)
{
    var result = "none";
    return result;
}

function getAlign(range)
{
var result = "left";
try{
result = range.paragraphAttributes.justification.toString().toLowerCase().substring(14);       
       }catch(e){}
    return result;
}

function getTextDecoration(range)
{
var result = "none";

    if(range.characterAttributes.underline)
    result = "underline";
 

    if(range.characterAttributes.strikeThrough)
    result = "line-through";
   
    return result;
}



function getColor(color)
{
    var result = [0,0,0];
    //textStyleNode.@color = r+"r"+g+"g"+b+"b0a";
        try {
            switch (color.typename) {
                case "GrayColor":
                    result = app.convertSampleColor(ImageColorSpace.GRAY, [color.gray], ImageColorSpace.RGB, ColorConvertPurpose.defaultpurpose);
                    break;
                case "RGBColor":
                    result = [color.red, color.green, color.blue];
                    break;
                case "CMYKColor":
                    result = app.convertSampleColor(ImageColorSpace.CMYK, [color.cyan, color.magenta, color.yellow, color.black], ImageColorSpace.RGB, ColorConvertPurpose.defaultpurpose);
                    break;
                case "SpotColor":
                    result = getTextColor(color.spot.color);
                    break;
                case "LabColor":
                    result = app.convertSampleColor(ImageColorSpace.LAB, [color.l, color.a, color.b], ImageColorSpace.RGB, ColorConvertPurpose.defaultpurpose);
                    break;
            }
        } catch (e) {}
        return result;
  
}

function getItalic(name)
{
        if(name.toLowerCase().indexOf("oblique")!=-1 || name.toLowerCase().indexOf("italic")!=-1)
        return "italic";
        
        return "normal";
}

function getFontWeight(style) {
    var weight = 400;
    try{
        style = style.split(" ")[0];

        if (style == "Black" || style=="Heavy") {
          weight = 900;
        } else if (style=="ExtraBold" || style=="UltraBold") {
          weight = 800;
        } else if (style=="Bold") {
          weight = 700;
        } else if (style=="SemiBold" || style=="DemiBold") {
          weight = 600;
        } else if (style=="Medium") {
          weight = 500;
        } else if (style=="Regular" || style=="Book" || style=="Regular" || style=="Demi" || style=="Text") {
          weight = 400;
        } else if (style=="Light") {
          weight = 300;
        } else if (style=="UltraLight" || style=="ExtraLight") {
          weight = 200;
        } else if (style=="Thin") {
          weight = 100;
        }
    }catch(e){}
    return weight;
}

function isAllCaps(range)
{
    try{
        var textCase = range.characterAttributes.capitalization;
        return textCase==FontCapsOption.ALLCAPS;
    }catch(e){}
        
    return false;
}




function boundsArray(textFrame) {
    var bounds = textFrame.geometricBounds;
/*
    if(textFrame.typename == "TextFrame") {
        try {
            var dup = textFrame.duplicate();
            var target = dup.createOutline();
            bounds = target.visibleBounds;
            target.remove();
        } catch(e) {}
    }
*/
    return bounds;
}

function getCoords(obj) {
    var result = {};

    result.x = obj.geometricBounds[0];
    result.y = obj.geometricBounds[1];
    result.x2 = obj.geometricBounds[2];
    result.y2 = obj.geometricBounds[3];
    result.center = {};
    result.center.x = obj.geometricBounds[2] / 2 + obj.geometricBounds[0] / 2;
    result.center.y = obj.geometricBounds[3] / 2 + obj.geometricBounds[1] / 2;
    result.width = Math.abs(obj.geometricBounds[2] - obj.geometricBounds[0]);
    result.height = Math.abs(obj.geometricBounds[3] - obj.geometricBounds[1]);

    return result;
}

function getVisibleCoords(obj) {
    var result = new Object();
    var bounds = boundsArray(obj);
    result.x = bounds[0];
    result.y = bounds[1];
    result.x2 = bounds[2];
    result.y2 = bounds[3];

    result.center = new Object();
    result.center.x = bounds[2] / 2 + bounds[0] / 2;
    result.center.y = bounds[3] / 2 + bounds[1] / 2;
    result.width = Math.abs(bounds[2] - bounds[0]);
    result.height = Math.abs(bounds[3] - bounds[1]);

    return result;
}

function itemInArtboard(item,rect)
{
    
    //(RectA.X1 < RectB.X2 && RectA.X2 > RectB.X1 &&
//    RectA.Y1 < RectB.Y2 && RectA.Y2 > RectB.Y1)

        var bb = item.geometricBounds;
        
        if(bb[0] < rect[2] && bb[2] > rect[0] && bb[3] < rect[1] && bb[1] > rect[3])
        return true;
        
        return false;
}

function getLayerChildren(layerRef,artRect)
{
    var result = [];
    var allLayerItems = layerRef.pageItems;

           for(var i=0;i<layerRef.pageItems.length;i++)
    try{
   
        var currLayer = layerRef.pageItems[i];

        if(currLayer.parent.typename == "CompoundPathItem") continue;
        
        if(itemInArtboard(currLayer,artRect))
        {
             
            var currResult =  {itemRef:currLayer,
       name:currLayer.name,
       type:currLayer.typename,
        locked:currLayer.locked,
       visible:!currLayer.hidden,
      zIndex:currLayer.absoluteZOrderPosition
       };
  
            if(currLayer.typename == "GroupItem") currResult.children = getGroupChildren(currLayer);
       
            result.push(currResult);
   currLayer.locked = false;
   currLayer.hidden = false;
   
          
            
        }
    }catch(e){log(e)}
    
    
    var allLayers = layerRef.layers;
    
    for(var i=0;i<allLayers.length;i++)
    {
        var currLayer = allLayers[i];
        result.push({itemRef:currLayer,
       name:currLayer.name,
       type:currLayer.typename,
        locked:currLayer.locked,
       visible:currLayer.visible,
       zIndex:currLayer.absoluteZOrderPosition,
       children:getLayerChildren(currLayer,artRect)
       });      
   
    currLayer.locked = false;
   currLayer.visible = true;
   
    }


  
    return result.sort(sortByZIndex);
}


function getGroupChildren(layerRef)
{
    
    log("getGroupChildren");
    var result = [];
    var allLayerItems = layerRef.pageItems;

           for(var i=0;i<layerRef.pageItems.length;i++)
    try{
        var currLayer = layerRef.pageItems[i];
        if(currLayer.parent.typename == "CompoundPathItem") continue;

            var currResult =  {itemRef:currLayer,
       name:currLayer.name,
       type:currLayer.typename,
        locked:currLayer.locked,
       visible:!currLayer.hidden,
      zIndex:currLayer.absoluteZOrderPosition
       };
 
            if(currLayer.typename == "GroupItem") currResult.children = getGroupChildren(currLayer);
            
            result.push(currResult);
   currLayer.locked = false;
   currLayer.hidden = false;
   
          
            
    }catch(e){log(e)}
    
   
  
    return result.sort(sortByZIndex);
}

function sortByZIndex(a,b)
{
return a.zIndex-b.zIndex;    
}


    function getOrientation(artRect)
    {
        
      
    var artboardWidth = (artRect[2]-artRect[0]);
    var artboardHeight = (artRect[1]-artRect[3]);
    
        var result = "PORTRAIT";
        if(artboardWidth>artboardHeight) result = "LANDSCAPE";
        return result;
    }

    function currDate()
    {
    var today = new Date();
    return today.toLocaleString();
    }



    function isFolder(obj)
    {
        var result = false;
        if (obj.constructor.name === "Folder") result = true;
        return result;
    }

    function isFile(obj)
    {
	var result = false;
	if (obj.constructor.name === "File" && !obj.hidden) result = true;
	return result;
}


function log(msg)
{
if (debug) 
try{
$.writeln(msg);    
$.writeln(msg.toSource()); 
}catch(e){}
}

function createNewUUID() {
  return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
    var r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
    return v.toString(16);
  });
}


function calculateValidName(originalName){
  var name = originalName.replace(/ /g,"_");
  name = name.replace(/([^a-zA-Z0-9_-])/g,"");
  name = name.replace(/(^[^a-zA-Z]*)/,"");
  if (name === "") 
  	name="Layer";
  return getValidUniqueName(name);
}

function getValidUniqueName(originalName,calculatedName,calculatedIndex){
  if(!calculatedName){
    calculatedName = originalName;
  }

  if(mapID[calculatedName]){
    if(calculatedIndex){
      calculatedIndex++;
    }else{
      calculatedIndex = 1;
    }
    return getValidUniqueName(originalName,originalName+"_"+calculatedIndex,calculatedIndex);
  }
  mapID[calculatedName] = originalName;
  return calculatedName;
}

function removeInvalidCharacters(string)
{
    // remove everything forbidden by XML 1.0 specifications, plus the unicode replacement character U+FFFD
    var regex = /((?:[\0-\x08\x0B\f\x0E-\x1F\uFFFD\uFFFE\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF]))/g;
    string = string.replace(regex, "");
 	// remove everything not suggested by XML 1.0 specifications
    regex = new RegExp(
            "([\\x7F-\\x84]|[\\x86-\\x9F]|[\\uFDD0-\\uFDEF]|(?:\\uD83F[\\uDFFE\\uDFFF])|(?:\\uD87F[\\uDF"+
            "FE\\uDFFF])|(?:\\uD8BF[\\uDFFE\\uDFFF])|(?:\\uD8FF[\\uDFFE\\uDFFF])|(?:\\uD93F[\\uDFFE\\uD"+
            "FFF])|(?:\\uD97F[\\uDFFE\\uDFFF])|(?:\\uD9BF[\\uDFFE\\uDFFF])|(?:\\uD9FF[\\uDFFE\\uDFFF])"+
            "|(?:\\uDA3F[\\uDFFE\\uDFFF])|(?:\\uDA7F[\\uDFFE\\uDFFF])|(?:\\uDABF[\\uDFFE\\uDFFF])|(?:\\"+
            "uDAFF[\\uDFFE\\uDFFF])|(?:\\uDB3F[\\uDFFE\\uDFFF])|(?:\\uDB7F[\\uDFFE\\uDFFF])|(?:\\uDBBF"+
            "[\\uDFFE\\uDFFF])|(?:\\uDBFF[\\uDFFE\\uDFFF])(?:[\\0-\\t\\x0B\\f\\x0E-\\u2027\\u202A-\\uD7FF\\"+
            "uE000-\\uFFFF]|[\\uD800-\\uDBFF][\\uDC00-\\uDFFF]|[\\uD800-\\uDBFF](?![\\uDC00-\\uDFFF])|"+
            "(?:[^\\uD800-\\uDBFF]|^)[\\uDC00-\\uDFFF]))", "g");
        string = string.replace(regex, "");
    return string;

}

